@@ -95,20 +95,19 @@ void qiv_display_text_window(qiv_image *q, const char *infotextdisplay,
                              const char *strs[], const char *continue_msg)
 {
   int temp, text_w = 0, text_h, i, maxlines;
-  int x, y, width, height, text_left;
+  int width, height, text_left;
 
   int ascent;
   int descent;
+  cairo_t *cr;
 
   ascent  = PANGO_PIXELS(pango_font_metrics_get_ascent(metrics));
   descent = PANGO_PIXELS(pango_font_metrics_get_descent(metrics));
 
   if (fullscreen) {
-    x = y = 0;
     width = monitor[q->mon_id].width;
     height = monitor[q->mon_id].height;
   } else {
-    x = y = 0;
     width = q->win_w;
     height = q->win_h;
   }
@@ -135,27 +134,29 @@ void qiv_display_text_window(qiv_image *q, const char *infotextdisplay,
   text_left = width/2 - text_w/2 - 4;
   if (text_left < 2)  text_left = 2;            /* if window/screen is smaller than text */
 
-  gdk_draw_rectangle(q->win, q->bg_gc, y,
-                     x + text_left,
-                     y + height/2 - text_h/2 - 4,
-                     x + text_w + 7, text_h + 7);
-  gdk_draw_rectangle(q->win, q->status_gc, y + 1,
-                     x + text_left + 1,
-                     y + height/2 - text_h/2 - 3,
-                     x + text_w + 6, text_h + 6);
+  cr = gdk_cairo_create (q->win);
+
+  set_cairo_color(cr, 0);
+  cairo_rectangle(cr, text_left, height/2 - text_h/2 - 4, text_w+7, text_h+7);
+  cairo_fill (cr);
+
+  set_cairo_color(cr, STATUSBAR_BG);
+  cairo_rectangle(cr, text_left + 1, height/2 - text_h/2 - 3, text_w+5, text_h+5);
+  cairo_fill (cr);
+
+  set_cairo_color(cr, 0);
   for (i = 0; strs[i] && i < maxlines; i++) {
        pango_layout_set_text(layout, strs[i], -1);
-       gdk_draw_layout (q->win, q->text_gc, x + text_left + 4, y + height/2 - text_h/2  +
-                  i * (ascent + descent), layout);
+       cairo_move_to (cr,  text_left + 4, height/2 - text_h/2  + i * (ascent + descent));
+       pango_cairo_show_layout(cr, layout);
   }
 
   /* Display Push Any Key... message */
   pango_layout_set_text(layout, continue_msg, -1);
   pango_layout_get_pixel_size (layout, &temp, NULL);
-  gdk_draw_layout (q->win, q->text_gc,
-                   x + width/2 - temp/2,
-                   y + height/2 - text_h/2 - descent + (i+1) * (ascent + descent),
-                   layout);
+  cairo_move_to (cr, width/2 - temp/2, height/2 - text_h/2 - descent + (i+1)*(ascent + descent));
+  pango_cairo_show_layout(cr, layout);
+  cairo_destroy(cr);
   displaying_textwindow = TRUE;
 
   /* print also on console */
@@ -299,7 +300,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
     case GDK_BUTTON_RELEASE:
       exit_slideshow = TRUE;
       switch (ev->button.button) {
-        /* [tw]: I think buttons 4-7 don't work in GDK-2.0 as button events,
+        /* [tw]: buttons 4-7 don't work in GDK-2.0 as button events,
          *       they are now GDK_SCROLL events */
         case 1:        /* left button pressed */
           displaying_textwindow = FALSE;
@@ -319,8 +320,6 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             }
             q->drag = 0;
           }
-        case 5:        /* scroll wheel down emulated by button 5 */
-          displaying_textwindow = FALSE;
           goto next_image;
         case 8:
           displaying_textwindow = FALSE;
@@ -334,7 +333,6 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
           qiv_exit(0);
           break;
         case 3:        /* right button pressed */
-        case 4:        /* scroll wheel up emulated by button 4 */
           displaying_textwindow = FALSE;
           goto previous_image;
           break;
@@ -342,19 +340,80 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
       break;
 
     case GDK_SCROLL:
+      displaying_textwindow = FALSE;
       switch (ev->scroll.direction) {
         case GDK_SCROLL_UP:
-          displaying_textwindow = FALSE;
-	  goto previous_image;
+          if (fullscreen) {
+            if (ev->key.state & GDK_SHIFT_MASK) {
+              move_step = (MIN(monitor[q->mon_id].height, q->win_h) / STEP);
+            } else {
+              move_step = (q->win_h / 100);
+            }
+            if (move_step < 10)
+              move_step = 10;
+
+            qiv_drag_image(q, q->win_x, q->win_y + move_step, "(Moving down)", "(Cannot move further down)");
+          } else {
+            snprintf(infotext, sizeof infotext, "(5 pictures back)");
+            next_image(-5);
+            if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
+            qiv_load_image(q);
+          }
+	  break;
         case GDK_SCROLL_DOWN:
-          displaying_textwindow = FALSE;
-	  goto next_image;
+          if (fullscreen) {
+            if (ev->key.state & GDK_SHIFT_MASK) {
+              move_step = (MIN(monitor[q->mon_id].height, q->win_h) / STEP);
+            } else {
+              move_step = (q->win_h / 100);
+            }
+            if (move_step < 10)
+              move_step = 10;
+
+            qiv_drag_image(q, q->win_x, q->win_y - move_step, "(Moving up)", "(Cannot move further up)");
+          } else {
+            snprintf(infotext, sizeof infotext, "(5 pictures forward)");
+            next_image(5);
+            if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
+            qiv_load_image(q);
+          }
+	  break;
         case GDK_SCROLL_LEFT:
-          displaying_textwindow = FALSE;
-          goto zoom_out;
+          if (fullscreen) {
+            if (ev->key.state & GDK_SHIFT_MASK) {
+              move_step = (MIN(monitor[q->mon_id].width, q->win_w) / STEP);
+            } else {
+              move_step = (q->win_w / 100);
+            }
+            if (move_step < 10)
+              move_step = 10;
+
+            qiv_drag_image(q, q->win_x + move_step, q->win_y, "(Moving right)", "(Cannot move further right)");
+          } else {
+            snprintf(infotext, sizeof infotext, "(Previous picture)");
+            next_image(-1);
+            if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
+            qiv_load_image(q);
+          }
+	  break;
         case GDK_SCROLL_RIGHT:
-          displaying_textwindow = FALSE;
-          goto zoom_in;
+          if (fullscreen) {
+            if (ev->key.state & GDK_SHIFT_MASK) {
+              move_step = (MIN(monitor[q->mon_id].width, q->win_w) / STEP);
+            } else {
+              move_step = (q->win_w / 100);
+            }
+            if (move_step < 10)
+              move_step = 10;
+
+            qiv_drag_image(q, q->win_x - move_step, q->win_y, "(Moving left)", "(Cannot move further left)");
+          } else {
+            snprintf(infotext, sizeof infotext, "(Next picture)");
+            next_image(1);
+            if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
+            qiv_load_image(q);
+          }
+	  break;
         default:
 	  break;
       }
@@ -593,7 +652,27 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             comment_window ^= 1;
             snprintf(infotext, sizeof infotext, comment_window ?
                      "(Comments: on)" : "(Comments: off)");
-            update_image(q, MIN_REDRAW);
+            if (fullscreen) {
+              /* REDRAW or MIN_REDRAW is not enough if the text window is outside the image area */
+              update_image(q, FULL_REDRAW);
+            } else {
+              update_image(q, MIN_REDRAW);
+            }
+            break;
+
+            /* artist name on/off  */
+
+          case 'A':
+            exit_slideshow = FALSE;
+            artist_window ^= 1;
+            snprintf(infotext, sizeof infotext, artist_window ?
+                     "(Artist: on)" : "(Artist: off)");
+            if (fullscreen) {
+              /* REDRAW or MIN_REDRAW is not enough if the text window is outside the image area */
+              update_image(q, FULL_REDRAW);
+            } else {
+              update_image(q, MIN_REDRAW);
+            }
             break;
 
             /* Slide show on/off */
@@ -626,8 +705,12 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
               qiv_drag_image(q, q->win_x + move_step, q->win_y, "(Moving right)", "(Cannot move further right)");
             } else {
-              snprintf(infotext, sizeof infotext, "(Moving works only in fullscreen mode)");
-              fprintf(stdout, "qiv: Moving works only in fullscreen mode\n");
+//              snprintf(infotext, sizeof infotext, "(Moving works only in fullscreen mode)");
+//              fprintf(stdout, "qiv: Moving works only in fullscreen mode\n");
+              snprintf(infotext, sizeof infotext, "(Previous picture)");
+              next_image(-1);
+              if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
+              qiv_load_image(q);
             }
             break;
 
@@ -647,8 +730,12 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
               qiv_drag_image(q, q->win_x - move_step, q->win_y, "(Moving left)", "(Cannot move further left)");
             } else {
-              snprintf(infotext, sizeof infotext, "(Moving works only in fullscreen mode)");
-              fprintf(stdout, "qiv: Moving works only in fullscreen mode\n");
+//              snprintf(infotext, sizeof infotext, "(Moving works only in fullscreen mode)");
+//              fprintf(stdout, "qiv: Moving works only in fullscreen mode\n");
+              snprintf(infotext, sizeof infotext, "(Next picture)");
+              next_image(1);
+              if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
+              qiv_load_image(q);
             }
             break;
 
@@ -1085,7 +1172,6 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
           case '7':
           case '8':
           case '9':
-          case 'A':
           case 'D':
 #ifndef HAVE_EXIF
           case 'E':
