@@ -1,6 +1,6 @@
 /*
   Module       : options.c
-  Purpose      : Read and evaluate commandline options
+  Purpose      : Read and evaluate command line options
   More         : see qiv README
   Homepage     : http://qiv.spiegl.de/
   Original     : http://www.klografx.net/qiv/
@@ -26,8 +26,10 @@ extern int optind, opterr, optopt;
 /* put longopt-only options to non ascii values */
 #define LONGOPT_VIKEYS   256
 #define LONGOPT_TRASHBIN 257
+#define LONGOPT_ARTISTIGNORE 258   /* do NOT show artist name if it is this string */
+#define LONGOPT_HIGHDPIFACTOR 259  /* resize everything by this factor */
 
-static char *short_options = "ab:c:Cd:efg:hilLmno:pq:rstuvw:xyzA:BDF:GIJKMNPRSTW:X:Y:Z:";
+static char *short_options = "ab:c:Cd:eEfg:hilLmno:pq:rstuvw:xyzA:BDF:GIJKMNPRSTW:X:Y:Z:";
 static struct option long_options[] =
 {
     {"do_grab",          0, NULL, 'a'},
@@ -61,6 +63,7 @@ static struct option long_options[] =
     {"browse",           0, NULL, 'B'},
     {"cycle",            0, NULL, 'C'},
     {"no_sort",          0, NULL, 'D'},
+    {"showArtistName"  , 0, NULL, 'E'},
     {"file",             1, NULL, 'F'},
     {"disable_grab",     0, NULL, 'G'},
     {"statusbar",        0, NULL, 'I'},
@@ -80,6 +83,8 @@ static struct option long_options[] =
 #endif
     {"trashbin",         0, NULL, LONGOPT_TRASHBIN},
     {"vikeys",           0, NULL, LONGOPT_VIKEYS},
+    {"artist_ignore",    1, NULL, LONGOPT_ARTISTIGNORE},
+    {"highDPIfactor",    1, NULL, LONGOPT_HIGHDPIFACTOR},
     {0,                  0, NULL, 0}
 };
 
@@ -248,7 +253,7 @@ void options_read(int argc, char **argv, qiv_image *q)
                 if ((q->mod.contrast<0) || (q->mod.contrast>512))
                     usage(argv[0],1);
                 break;
-            case 'd': delay=(int) (atof (optarg) * 1000);
+            case 'd': delay=(int) (checked_atof(optarg) * 1000);
                 if (delay < 0) {
                     g_print("Error: %s is an invalid slide show delay.\n",optarg);
                     exit(1);
@@ -313,6 +318,8 @@ void options_read(int argc, char **argv, qiv_image *q)
                 break;
             case 'D': need_sort = 0;
                 break;
+            case 'E': artist_window=1;
+                break;
             case 'F': if(rreadfile(optarg) < 0) {
                   g_print("Error: %s could not be opened: %s.\n",optarg, strerror(errno));
                   gdk_exit(1);
@@ -354,8 +361,19 @@ void options_read(int argc, char **argv, qiv_image *q)
             case LONGOPT_TRASHBIN:
                 trashbin=1;
                 break;
-            case LONGOPT_VIKEYS: vikeys=1;
-              break;
+            case LONGOPT_VIKEYS:
+                vikeys=1;
+                break;
+            case LONGOPT_ARTISTIGNORE:
+                artist_ignore=optarg;
+                break;
+            case LONGOPT_HIGHDPIFACTOR:
+                highDPIfactor=checked_atof(optarg);
+                if (highDPIfactor <= 0) {
+                    g_print("Error: %s is not a valid zoom factor.\n",optarg);
+                    exit(1);
+                }
+                break;
             case 0:
             case '?': usage(argv[0], 1);
                       gdk_exit(0);
@@ -395,7 +413,7 @@ void options_read(int argc, char **argv, qiv_image *q)
         char *fn;
         int i, p;
         for (i=0;i<images;i++) { /* simple insertion sort, fine for small num */
-          p = (int)(((float)rand()/RAND_MAX) * (images-i)) + i;
+          p = (int)(((float)rand()/(float)RAND_MAX) * (images-i)) + i;
           fn = image_names[i];
           image_names[i] = image_names[p];
           image_names[p] = fn;
