@@ -26,16 +26,18 @@
 #include <X11/extensions/xf86vmode.h> // for XF86VidModeGetModeLine
 */
 
-#define VERSION "2.3.2"
-#define VERSION_FULL "QIV - Quick Image Viewer v2.3.2 - http://qiv.spiegl.de/"
+#define VERSION "2.3.3"
+#define VERSION_FULL "QIV - Quick Image Viewer v2.3.3 - http://qiv.spiegl.de/"
 #define TRASH_DIR ".qiv-trash"
 #define SELECT_DIR ".qiv-select"
 #define SLIDE_DELAY 3000 /* milliseconds */
 #define IMAGE_BG "black"
-#define STATUSBAR_BG "#FFB900"
+#define STATUSBAR_BG 0xFFB900
 #define STATUSBAR_FS 8  /* default fontsize if none is given */
-#define COMMENT_BG "#DDDDDD"
+#define COMMENT_BG 0xDDDDDD
 #define COMMENT_FS 20  /* default fontsize if none is given */
+#define ARTIST_BG 0xBBBBBB
+#define ARTIST_FS 10  /* default fontsize if none is given */
 #define ERROR_BG "#0000FF"
 #define DEFAULT_BRIGHTNESS 256
 #define DEFAULT_CONTRAST 256
@@ -62,9 +64,6 @@ typedef struct _qiv_image {
   gint win_x, win_y, win_w, win_h, mon_id; /* window co-ordinates */
   gint orig_w, orig_h; /* Size of original image in pixels */
   GdkGC *bg_gc;     /* image window background */
-  GdkGC *text_gc;   /* statusbar text color */
-  GdkGC *status_gc; /* statusbar background */
-  GdkGC *comment_gc; /* comment background */
 
   /* These are used to work out how to redraw in fullscreen mode */
   gint win_ox, win_oy, win_ow, win_oh; /* coordinates currently drawn at */
@@ -78,6 +77,7 @@ typedef struct _qiv_image {
   gchar win_title[BUF_LEN];
   gint text_len, text_w, text_h;
   gint comment_w, comment_h;
+  gint artist_w, artist_h;
 } qiv_image;
 
 typedef struct _qiv_mgl {
@@ -109,9 +109,7 @@ extern GdkFont          *text_font;
 extern GdkColormap      *cmap;
 extern char             *image_bg_spec;
 extern GdkColor         image_bg;
-extern GdkColor         text_bg;
 extern GdkColor         error_bg;
-extern GdkColor         comment_bg;
 extern int              images;
 extern char             **image_names;
 extern int              image_idx;
@@ -127,6 +125,10 @@ extern PangoLayout     *layoutComment;
 extern PangoFontDescription *fontdescComment;
 extern PangoFontMetrics *metricsComment;
 extern char             *comment;
+extern PangoLayout     *layoutArtist;
+extern PangoFontDescription *fontdescArtist;
+extern PangoFontMetrics *metricsArtist;
+extern char             *artist;
 extern gint             jpeg_prog;
 extern off_t            file_size;
 
@@ -145,6 +147,7 @@ extern int     maxpect;
 extern int     statusbar_fullscreen;
 extern int     statusbar_window;
 extern int     comment_window;
+extern int     artist_window;
 extern int     slide;
 extern int     scale_down;
 extern int     recursive;
@@ -167,6 +170,8 @@ extern int     autorotate;
 extern int     rotation;
 extern int     vikeys;
 extern int     trashbin;
+extern char    *artist_ignore;
+extern float   highDPIfactor;
 
 extern const char   *helpstrs[], **helpkeys, *image_extensions[];
 
@@ -215,6 +220,9 @@ extern void hide_cursor(qiv_image *q);
 extern void show_cursor(qiv_image *q);
 extern void setup_magnify(qiv_image *, qiv_mgl *); // [lc]
 extern void update_magnify(qiv_image *, qiv_mgl *,int, gint, gint); // [lc]
+static inline void set_cairo_color(cairo_t *cr, u_int32_t col) {
+  cairo_set_source_rgb(cr, (col >> 16)/256.0, ((col >> 8) & 0xff)/256.0, (col & 0xff)/256.0);
+}
 
 /* event.c */
 
@@ -235,6 +243,7 @@ extern void run_command(qiv_image *, char *, char *, int *, const char ***);
 extern void finish(int);
 extern void next_image(int);
 extern int checked_atoi(const char *);
+extern float checked_atof(const char *);
 extern void usage(char *, int);
 extern void show_help(char *, int);
 extern int get_random(int, int, int);
@@ -248,6 +257,7 @@ extern int rreadfile(const char *);
 extern int find_image(int images, char **image_names, char *name);
 #ifdef HAVE_EXIF
 extern char **get_exif_values(char *filename);
+extern char *get_exif_artist(char *filename);
 #endif
 void dpms_check();
 void dpms_enable();
