@@ -486,6 +486,19 @@ int checked_atoi (const char *s)
     return num;
 }
 
+float checked_atof (const char *s)
+{
+    char *endptr;
+    float num = strtof(s, &endptr);
+
+    if (endptr == s || *endptr != '\0') {
+	g_print("Error: %s is not a valid floating point number.\n", s);
+	gdk_exit(1);
+    }
+
+    return num;
+}
+
 void usage(char *name, int exit_status)
 {
     g_print("qiv (Quick Image Viewer) v%s\n"
@@ -518,6 +531,7 @@ void show_help(char *name, int exit_status)
           "    --disable_grab, -G     Disable pointer/kbd grab in fullscreen mode\n"
           "    --fixed_width, -w x    Window with fixed width x\n"
           "    --fixed_zoom, -W x     Window with fixed zoom factor (percentage x)\n"
+          "    --highDPIfactor FLOAT  Resize factor to compensate for high (or also low) DPI screens\n"
           "    --fullscreen, -f       Use fullscreen window on start-up\n"
           "    --gamma, -g x          Set gamma to x (-32..32)\n"
           "    --help, -h             This help screen\n"
@@ -529,6 +543,9 @@ void show_help(char *name, int exit_status)
           "    --no_filter, -n        Do not filter images by extension\n"
           "    --no_statusbar, -i     Disable statusbar\n"
           "    --statusbar, -I        Enable statusbar\n"
+          "    --showJPEGcomments, -J Enable display of JPEG comments\n"
+          "    --showArtistName, -E   Enable display of artist (author) name\n"
+          "    --artist_ignore x      Do NOT show artist name if it is equal to x\n"
           "    --no_sort, -D          Do not apply any sorting to the list of files\n"
           "    --numeric_sort, -N     Sort filenames with numbers intuitively\n"
           "    --root, -x             Set centered desktop background and exit\n"
@@ -608,7 +625,7 @@ int get_random(int replace, int num, int direction)
 
       for (n=0;n<num;n++)   /* simple insertion sort, fine for num small */
 	{
-	  p=(int)(((float)rand()/RAND_MAX) * (num-n)) + n ; /* n <= p < num */
+	  p=(int)(((float)rand()/(float)RAND_MAX) * (num-n)) + n ; /* n <= p < num */
 	  q=rindices[n];
 	  rindices[n]=rindices[p]; /* Switch the two numbers to make random order */
 	  rindices[p]=q;
@@ -756,6 +773,7 @@ gboolean qiv_watch_file (gpointer data)
           if(time(NULL)-statbuf.st_mtime > 0)
           {
               reload_image(q);
+              check_size(q, FALSE);
               update_image(q, REDRAW);
           }
   }
@@ -866,7 +884,7 @@ char *get_icc_profile(char *filename)
 
         i++;
       }
-      /* copy jpeg comment here to be printed out when exif data is displayed. */
+      /* copy jpeg comment here to be printed out when EXIF data or comment window is displayed. */
       else if(marker->marker==JPEG_COM)
       {
         comment=calloc(1+marker->data_length,1);
@@ -913,7 +931,7 @@ char *get_icc_profile(char *filename)
   /* is pic a tiff?*/
   else if((pic_tst[0] == pic_tst[1]) && ((pic_tst[0] == 0x49)||(pic_tst[0] == 0x4d)) && (pic_tst[2] == 0x2a) &&  (pic_tst[3] == 0x00))
   {
-    uint16 count;
+    uint16_t count;
     unsigned char *data;
 
     fclose(infile);
@@ -952,6 +970,7 @@ char **get_exif_values(char *filename)
     ExifTag tag;
     ExifIfd ifd;
   } tags[] = {
+    {EXIF_TAG_ARTIST, EXIF_IFD_0},
     {EXIF_TAG_MAKE, EXIF_IFD_0},
     {EXIF_TAG_MODEL, EXIF_IFD_0},
     {EXIF_TAG_DOCUMENT_NAME, EXIF_IFD_0},
@@ -960,7 +979,6 @@ char **get_exif_values(char *filename)
     {EXIF_TAG_SOFTWARE, EXIF_IFD_0},
     {EXIF_TAG_DATE_TIME, EXIF_IFD_0},
     {EXIF_TAG_DATE_TIME_ORIGINAL, EXIF_IFD_EXIF},
-    {EXIF_TAG_ARTIST, EXIF_IFD_0},
     {EXIF_TAG_ORIENTATION, EXIF_IFD_0},
     {EXIF_TAG_JPEG_PROC, EXIF_IFD_1},
     {EXIF_TAG_FLASH, EXIF_IFD_EXIF},
@@ -993,15 +1011,15 @@ char **get_exif_values(char *filename)
   ed = exif_data_new_from_file(filename);
   if(ed)
   {
-    /* one too much to make sure the last one will allways be NULL
-       size of tags + gpstags(3) + extra lines(3) + 1 */ 
+    /* one too much to make sure the last one will always be NULL
+       size of tags + gpstags(3) + extra lines(3) + 1 */
     exif_lines=calloc(sizeof(tags)/sizeof(tags[0]),sizeof(char*)+3+3+1);
     for(i=0; i < sizeof(tags)/sizeof(tags[0]); i++)
     {
       entry=exif_content_get_entry( ed->ifd[tags[i].ifd], tags[i].tag);
       if(entry)
       {
-        line=malloc(256); 
+        line=malloc(256);
         exif_entry_get_value (entry, buffer, 255);
         snprintf(line, 255, "%-21s: %s\n", exif_tag_get_name_in_ifd(tags[i].tag,tags[i].ifd), buffer);
         exif_lines[j]=line;
@@ -1013,7 +1031,7 @@ char **get_exif_values(char *filename)
       entry=exif_content_get_entry( ed->ifd[EXIF_IFD_GPS], gps_tags[i]);
       if(entry)
       {
-        line=malloc(256); 
+        line=malloc(256);
         exif_entry_get_value (entry, buffer, 255);
         if((entry=exif_content_get_entry( ed->ifd[EXIF_IFD_GPS], i+1)))
         {
@@ -1053,6 +1071,28 @@ char **get_exif_values(char *filename)
 
   return exif_lines;
 }
+
+char *get_exif_artist(char *filename)
+{
+  ExifData *ed;
+  ExifEntry *entry;
+  char *exif_artist=NULL;
+  char buffer[255];
+
+  ed = exif_data_new_from_file(filename);
+  if(ed)
+  {
+    entry=exif_content_get_entry( ed->ifd[ EXIF_IFD_0], EXIF_TAG_ARTIST );
+    if(entry)
+    {
+      exif_artist=malloc(255);
+      exif_entry_get_value (entry, buffer, 255);
+      snprintf(exif_artist, 255, "%s", buffer);
+    }
+  }
+  // printf("DEBUG: Artist: %s\n", exif_artist);
+  return exif_artist;
+}
 #endif
 
 /* The state of DPMS is checked at startup of qiv.
@@ -1078,7 +1118,7 @@ void dpms_check()
 void dpms_enable()
 {
   if(dpms_enabled)
-  { 
+  {
     DPMSEnable(GDK_DISPLAY());
   }
 }
@@ -1086,7 +1126,7 @@ void dpms_enable()
 void dpms_disable()
 {
   if(dpms_enabled)
-  { 
+  {
     DPMSDisable(GDK_DISPLAY());
   }
 }
